# Copyright (c) 2020 Mike Kinney

"""Common pytest code."""
import json
import pytest


@pytest.fixture
def mechcloudfile_one_entry():
    """Return one mechcloudfile entry."""
    return {
        'tophat': {
            'name': 'tophat',
            'hostname': 'tophat.example.com',
            'directory': '~/test1',
            'username': 'bob'
        }
    }


@pytest.fixture
def ssh_config():
    """Return one ssh_config."""
    return {
        "Host": "first",
        "User": "foo",
        "Port": "22",
        "UserKnownHostsFile": "/dev/null",
        "StrictHostKeyChecking": "no",
        "PasswordAuthentication": "no",
        "IdentityFile": 'blah',
        "IdentitiesOnly": "yes",
        "LogLevel": "FATAL",
    }


@pytest.fixture
def mechfile_one_entry():
    """Return one mechfile entry."""
    return {
        'first': {
            'name': 'first',
            'box': 'bento/ubuntu-18.04',
            'box_version': '201912.04.0'
        }
    }


@pytest.fixture
def mechfile_one_entry_without_box_version():
    """Return one mechfile entry."""
    return {
        'first': {
            'name': 'first',
            'box': 'bento/ubuntu-18.04'
        }
    }


@pytest.fixture
def mechfile_one_entry_with_file():
    """Return one mechfile entry."""
    return {
        'name': 'first',
        'box': 'bento/ubuntu-18.04',
        'box_version': '201912.04.0',
        'file': '/tmp/somefile.box'
    }


@pytest.fixture
def mechfile_one_entry_with_auth():
    """Return one mechfile entry with auth."""
    return {
        'first': {
            'name': 'first',
            'box': 'bento/ubuntu-18.04',
            'box_version': '201912.04.0',
            'auth': {
                'username': 'bob',
                'pub_key': 'some_pub_key_data'
            }
        }
    }


@pytest.fixture
def mechfile_one_entry_with_auth_and_mech_use():
    """Return one mechfile entry with auth that has mech_use."""
    return {
        'first': {
            'name': 'first',
            'box': 'bento/ubuntu-18.04',
            'box_version': '201912.04.0',
            'auth': {
                'username': 'bob',
                'mech_use': 'true',
                'pub_key': 'some_pub_key_data'
            }
        }
    }


@pytest.fixture
def mechfile_two_entries():
    """Return two mechfile entries."""
    return {
        'first': {
            'name': 'first',
            'box': 'bento/ubuntu-18.04',
            'box_version': '201912.04.0',
            'shared_folders': [
                {
                    "host_path": ".",
                    "share_name": "mech"
                }
            ],
            'url':
            'https://vagrantcloud.com/bento/boxes/ubuntu-18.04/'
            'versions/201912.04.0/providers/vmware_desktop.box'
        },
        'second': {
            'name': 'second',
            'box': 'bento/ubuntu-18.04',
            'box_version': '201912.04.0',
            'url':
            'https://vagrantcloud.com/bento/boxes/ubuntu-18.04/'
            'versions/201912.04.0/providers/vmware_desktop.box'
        }
    }


CATALOG = """{
    "description": "foo",
    "short_description": "foo",
    "name": "bento/ubuntu-18.04",
    "versions": [
        {
            "version": "aaa",
            "status": "active",
            "description_html": "foo",
            "description_markdown": "foo",
            "providers": [
                {
                    "name": "vmware_desktop",
                    "url": "https://vagrantcloud.com/bento/boxes/ubuntu-18.04/\
versions/aaa/providers/vmware_desktop.box",
                    "checksum": null,
                    "checksum_type": null
                }
            ]
        }
    ]
}"""
@pytest.fixture
def catalog():
    """Return a catalog."""
    return CATALOG


@pytest.fixture
def shell_provision_config():
    return [
        {
            "type": "shell",
            "path": "file1.sh",
            "args": [
                "a=1",
                "b=true",
            ],
        },
        {
            "type": "shell",
            "inline": "echo hello from inline"
        },
        {
            "type": "shell",
            "inline": "echo hello2 from inline",
            "args": []
        }
    ]


@pytest.fixture
def pyinfra_provision_config():
    return [
        {
            "type": "pyinfra",
            "path": "file1.py",
            "args": [
                "sudo=True",
                "b=1",
            ],
        },
        {
            "type": "pyinfra",
            "path": "file2.py",
        }
    ]


@pytest.fixture
def pyinfra_provision_http_config():
    return [
        {
            "type": "pyinfra",
            "path": "https://github.com/Fizzadar/pyinfra/blob/"
            "master/examples/docker_ce.py",
            "args": [
                "sudo=True"
            ],
        }
    ]


@pytest.fixture
def catalog_as_json():
    """Return a catalog as json."""
    return json.loads(CATALOG)


@pytest.fixture
def mech_add_arguments():
    """Return the default 'mech add' arguments."""
    return {
        '--force': False,
        '--box-version': None,
        '--name': None,
        '--box': None,
        '--add-me': None,
        '--use-me': None,
        '--provider': None,
        '<location>': None,
    }


@pytest.fixture
def mech_up_arguments():
    """Return the default 'mech up' arguments."""
    return {
        '--disable-provisioning': None,
        '--disable-shared-folders': None,
        '--gui': None,
        '--memsize': None,
        '--no-cache': None,
        '--no-nat': None,
        '--numvcpus': None,
        '--remove-vagrant': None,
        '<instance>': 'first',
    }


@pytest.fixture
def mech_box_arguments():
    """Return the default 'mech box' arguments."""
    return {
        '--force': False,
        '--box-version': None,
        '<provider>': None,
        '<location>': None,
    }


@pytest.fixture
def mech_init_arguments():
    """Return the default 'mech init' arguments."""
    return {
        '--force': False,
        '--box-version': None,
        '--name': None,
        '--box': None,
        '--add-me': None,
        '--use-me': None,
        '--provider': None,
        '<location>': None,
    }


class Helpers:
    @staticmethod
    def get_mock_data_written(a_mock):
        """Helper function to get the data written to a mocked file."""
        written = ''
        for call in a_mock.mock_calls:
            tmp = '{}'.format(call)
            if tmp.startswith('call().write('):
                line = tmp.replace("call().write('", '')
                line = line.replace("')", '')
                line = line.replace("\\n", '\n')
                written += line
        return written


@pytest.fixture
def helpers():
    """Helper functions for testing."""
    return Helpers



# Copyright (c) 2020 Mike Kinney

"""mech cloud tests"""
import re

from unittest.mock import patch
from pytest import raises

import mech.command
import mech.mech
import mech.vmrun
import mech.mech_instance


def test_mech_cloud_init_no_name():
    """Test init()."""
    arguments = {
        '--hostname': None,
        '--directory': None,
        '--username': None,
        '<cloud-instance>': None
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    with raises(AttributeError, match=r"Must provide a name for the cloud instance"):
        mc.init(arguments)


def test_mech_cloud_init_no_hostname():
    """test init()."""
    arguments = {
        '--hostname': None,
        '--directory': None,
        '--username': None,
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    with raises(SystemExit, match=r"A non-blank hostname is required"):
        mc.init(arguments)


def test_mech_cloud_init_no_directory():
    """test init()."""
    arguments = {
        '--hostname': 'somehost',
        '--directory': None,
        '--username': None,
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    with raises(SystemExit, match=r"A non-blank directory is required"):
        mc.init(arguments)


def test_mech_cloud_init_directory_with_spaces():
    """test init()."""
    arguments = {
        '--hostname': 'somehost',
        '--directory': 'some dir',
        '--username': 'someuser',
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    with raises(SystemExit, match=r"directory cannot contain spaces"):
        mc.init(arguments)


def test_mech_cloud_init_no_username():
    """test init()."""
    arguments = {
        '--hostname': 'somehost',
        '--directory': 'somedir',
        '--username': None,
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    with raises(SystemExit, match=r"A non-blank username is required"):
        mc.init(arguments)


@patch('mech.utils.save_mechcloudfile')
def test_mech_cloud_init(mock_save_mechcloudfile):
    """test init()."""
    arguments = {
        '--hostname': 'somehost',
        '--directory': 'somedir',
        '--username': 'someuser',
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    mc.init(arguments)
    mock_save_mechcloudfile.assert_called()


@patch('mech.utils.cloud_exists', return_value=False)
def test_mech_cloud_remove_does_not_exist(mock_cloud_exists,
                                          capfd):
    """test init()."""
    arguments = {
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    mc.remove(arguments)
    out, _ = capfd.readouterr()
    assert re.search(r'does not exist', out)
    mock_cloud_exists.assert_called()


@patch('mech.utils.cloud_exists', return_value=True)
@patch('mech.utils.remove_mechcloudfile_entry')
def test_mech_cloud_remove_exists(mock_cloud_exists,
                                  mock_remove_mechcloudfile_entry,
                                  capfd):
    """test init()."""
    arguments = {
        '<cloud-instance>': 'somecloud'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    mc.remove(arguments)
    out, _ = capfd.readouterr()
    assert re.search(r'Removed', out, re.MULTILINE)
    mock_cloud_exists.assert_called()
    mock_remove_mechcloudfile_entry.assert_called()


@patch('mech.utils.load_mechcloudfile')
def test_mech_cloud_list(mock_load_mechcloudfile, capfd,
                         mechcloudfile_one_entry):
    """test init()."""
    arguments = {}
    mc = mech.mech.MechCloud(arguments=arguments)
    mock_load_mechcloudfile.return_value = mechcloudfile_one_entry
    mc.list(arguments)
    out, _ = capfd.readouterr()
    assert re.search(r'mech clouds', out, re.MULTILINE)
    assert re.search(r'tophat.example.com', out, re.MULTILINE)
    assert re.search(r'~/test1', out, re.MULTILINE)
    assert re.search(r'bob', out, re.MULTILINE)


@patch('mech.utils.load_mechcloudfile')
def test_mech_cloud_upgrade_one_cloud(mock_load_mechcloudfile,
                                      mechcloudfile_one_entry):
    """test init()."""
    arguments = {
        '<cloud-instance>': 'tophat'
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    mock_load_mechcloudfile.return_value = mechcloudfile_one_entry
    with patch.object(mech.mech.MechCloudInstance, 'upgrade', return_value='some output'):
        mc.upgrade(arguments)


@patch('mech.utils.load_mechcloudfile')
def test_mech_cloud_upgrade_all_instances(mock_load_mechcloudfile,
                                          mechcloudfile_one_entry):
    """test init()."""
    arguments = {
        '<cloud-instance>': None
    }
    mc = mech.mech.MechCloud(arguments=arguments)
    mock_load_mechcloudfile.return_value = mechcloudfile_one_entry
    with patch.object(mech.mech.MechCloudInstance, 'upgrade', return_value='some output'):
        mc.upgrade(arguments)



# Copyright (c) 2020 Mike Kinney

"""Mech integration tests: provisioning tests using psk"""
import re
import subprocess

import pytest


from . import utils


@pytest.mark.vmware
@pytest.mark.int
def test_int_provision_using_psk():
    """Provision testing using psk."""

    test_dir = "tests/int/provision_using_psk/tmp"
    utils.cleanup_dir_and_vms_from_dir(test_dir)

    # copy files from parent dir
    command = "cp ../file* .; cp ../Mechfile ."
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stdout == ''
    assert stderr == ''
    assert results.returncode == 0

    # up (and remove vagrant user)
    command = "mech up -r"
    expected_lines = [r".first.*started",
                      r".second.*started",
                      r".third.*started",
                      r"Added auth",
                      r"first.*Provision 2 entries",
                      r"second.*Provision 3 entries",
                      r"Nothing",
                      r"Removing username"]
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    for line in expected_lines:
        print(line)
        assert re.search(line, stdout, re.MULTILINE)

    # ensure file exists
    command = 'mech ssh -c "ls -al /tmp/file1.txt" first'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("/tmp/file1.txt", stdout)
    assert results.returncode == 0

    # destroy
    command = "mech destroy -f"
    expected = "Deleting"
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    assert re.search(expected, stdout)

    # clean up at the end
    utils.cleanup_dir_and_vms_from_dir(test_dir)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016-2017 Kevin Chung
# Copyright (c) 2018 German Mendez Bravo (Kronuz)
# Copyright (c) 2020 Mike Kinney
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
"""MechCloud class"""

from __future__ import print_function, absolute_import


import logging


from . import utils
from .mech_command import MechCommand
from .mech_cloud_instance import MechCloudInstance

LOGGER = logging.getLogger(__name__)


class MechCloud(MechCommand):
    """
    Usage: mech cloud <subcommand> [<args>...]

    Notes:

        Mech Cloud is an easy way to expose resources for non-local use.

        If you have a spare desktop/laptop:

            1. Install VMware Workstation and/or Oracle VirtualBox, and

            2. Install pre-requisites ("python3.7+" and "virtualenv")

               For Ubuntu:
                 sudo apt-get install python3.7 python3-pip
                 sudo pip3 install virtualenv

               For Fedora:
                 sudo dnf install python37 python3-pip
                 sudo pip3 install virtualenv

        See https://github.com/Fizzadar/pyinfra/tree/master/examples/virtualbox
        to install virtualbox using `pyinfra`.

        Then add that host as a mech "cloud-instance" using the
        "mech cloud init" command.

        This would allow you to spin up/down virtual machines on that
        computer. For instance, if you have a could-instance called "top",
        you could init and start a VM on the remote computer using:
            mech -C top init bento/ubuntu-18.04
            mech -C top up

        The host's directory needs to have a virtual environment setup in
        "venv" and "mech" needs to be installed under that virtual
        environment, such as: "pip install mikemech". The 'mech cloud init'
        takes care of this for you.

        Virtualbox instances are "global". So, you can have only one
        instance named "first".

    Available subcommands:
        init              initialize the mech cloud instance
        remove            remove a mech cloud instance
        list              list the mech clould instances

    For help on any individual subcommand run `mech cloud <subcommand> -h`
    """

    def init(self, arguments):  # pylint: disable=no-self-use,unused-argument
        """
        Initialize the Mech Cloud configuration.

        The directory will be created with a python virtual environment (venv)
        and "mikemech" will be installed into it.

        Usage: mech cloud init [options] --hostname HOST --directory DIR <cloud-instance>

        Notes:
           The "mech init" operation can be run again with same cloud-instance.
           (The values would be updated in the Mechcloudfile).

           The directory should not contain spaces.

           If you use '~' in the directory value, be sure it is surrounded by
           single quotes because without quotes or if double quotes (") are used,
           then the '~' will be expanded locally and will not use the remote value.
              Example: --directory '~/mikemech'

        Options:
            --hostname HOST                  Hostname (resolvable hostname or ip)
            --directory DIR                  Directory on remote host where mech will be installed
            --username USER                  Username to use for ssh
            -h, --help                       Print this help
        """
        hostname = arguments['--hostname']
        directory = arguments['--directory']
        username = arguments['--username']
        cloud_instance = arguments['<cloud-instance>']

        inst = MechCloudInstance(cloud_instance)
        inst.set_hostname(hostname)
        inst.set_directory(directory)
        inst.set_username(username)
        inst.init()

    def remove(self, arguments):  # pylint: disable=no-self-use,unused-argument
        """
        Remove a Mech Cloud instance

        Usage: mech cloud remove [options] <cloud-instance>

        Options:
            -h, --help                       Print this help
        """
        cloud_instance = arguments['<cloud-instance>']
        if utils.cloud_exists(cloud_instance):
            utils.remove_mechcloudfile_entry(name=cloud_instance)
            print("Removed ({}) from mech cloud.".format(cloud_instance))
            print("Be sure to remove any running virtual machines.")
        else:
            print("Cloud ({}) does not exist in the Mechcloudfile.", cloud_instance)
    # allow 'mech cloud delete' as alias to 'mech cloud remove'
    delete = remove

    def list(self, arguments):  # pylint: disable=no-self-use,unused-argument
        """
        List Mech Clouds

        Usage: mech cloud list [options]

        Options:
            -h, --help                       Print this help
        """
        print('=== mech clouds: ===')
        clouds = self.cloud_instances()
        for cloud in clouds:
            mci = MechCloudInstance(cloud)
            mci.read_config(cloud)
            print(mci)
            print()
    # allow 'mech cloud ls' as alias to 'mech cloud list'
    ls = list

    def upgrade(self, arguments):  # pylint: disable=no-self-use,unused-argument
        """
        Upgrade 'pip' and 'mikemech' on the cloud instances.

        Usage: mech cloud upgrade [options] [<cloud-instance>]

        Options:
            -h, --help                       Print this help
        """
        cloud_instance = arguments['<cloud-instance>']

        if cloud_instance:
            instances = [cloud_instance]
        else:
            instances = self.cloud_instances()

        for cloud_name in instances:
            mci = MechCloudInstance(cloud_name)
            mci.read_config(cloud_name)
            mci.upgrade()



# Copyright (c) 2020 Mike Kinney

"""Mech integration tests: provisioning tests using virtualbox provider
   Note: 'no-nat' (bridged) networking must be used for the test that
   provisions using pyinfra because the ubuntu server needs to install
   using apt. (over the internet)
"""
import re
import subprocess

import pytest


from . import utils


@pytest.mark.virtualbox
@pytest.mark.int
def test_int_provision_virtualbox():
    """Provision testing."""

    test_dir = "tests/int/provision_virtualbox/tmp"
    utils.cleanup_dir_and_vms_from_dir(test_dir, names=['firstvb', 'secondvb',
                                                        'thirdvb', 'fourthvb'])

    # copy files from parent dir
    command = "cp ../file* .; cp ../Mechfile ."
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stdout == ''
    assert stderr == ''
    assert results.returncode == 0

    # up without provisioning and with no-nat
    command = "mech up --no-nat --disable-provisioning"
    expected_lines = [r".first.*started",
                      r".second.*started",
                      r".third.*started",
                      r".fourth.*started"]
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    for line in expected_lines:
        print(line)
        assert re.search(line, stdout, re.MULTILINE)

    # show provisioning
    command = "mech provision -s"
    expected_lines = [r"first.*Provision 2 entries",
                      r"second.*Provision 3 entries",
                      r"fourth.*Provision 1 entries",
                      r"Nothing"]
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    for line in expected_lines:
        print(line)
        assert re.search(line, stdout, re.MULTILINE)

    # ensure there is no file on firstvb
    command = 'mech ssh -c "ls -al /tmp/file1.txt" firstvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("No such file or directory", stdout)
    assert results.returncode != 0

    # ensure there is no file on secondvb (testing shell non-inline shell provisioning)
    command = 'mech ssh -c "ls -al /tmp/file1.sh.out" secondvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("No such file or directory", stdout)
    assert results.returncode != 0

    # ensure there is no file on secondvb (testing shell inline shell provisioning)
    command = 'mech ssh -c "ls -al /tmp/inline_test.out" secondvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("No such file or directory", stdout)
    assert results.returncode != 0

    # ensure the package 'npm' is not installed on fourthvb
    command = 'mech ssh -c "dpkg -l npm" fourthvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert re.search("no packages found matching npm", stdout)
    assert stderr == ''
    assert results.returncode != 0

    # provision
    command = "mech provision"
    expected_lines = [r"first.*Provision 2 entries",
                      r"second.*Provision 3 entries",
                      r"fourth.*Provision 1 entries",
                      r"Nothing"]
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    for line in expected_lines:
        print(line)
        assert re.search(line, stdout, re.MULTILINE)

    # ensure file exists now on firstvb
    command = 'mech ssh -c "ls -al /tmp/file1.txt" firstvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("/tmp/file1.txt", stdout)
    assert results.returncode == 0

    # ensure file exists now on secondvb from non-inline shell provisioning
    command = 'mech ssh -c "ls -al /tmp/file1.sh.out" secondvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("/tmp/file1.sh.out", stdout)
    assert results.returncode == 0

    # ensure file1.sh.out has the args
    command = """mech ssh -c "grep 'a=1 b=true' /tmp/file1.sh.out" secondvb"""
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("a=1 b=true", stdout)
    assert results.returncode == 0

    # ensure file exists now on secondvb from inline shell provisioning
    command = 'mech ssh -c "ls -al /tmp/inline_test.out" secondvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("/tmp/inline_test.out", stdout)
    assert results.returncode == 0

    # ensure package 'npm' *is* installed on fourthvb (should have been installed
    # as part of the pyinfra provisioning)
    command = 'mech ssh -c "dpkg -l npm" fourthvb'
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert re.search("npm", stdout)
    assert results.returncode == 0

    # ensure provisioning runs during "up"
    command = "mech up"
    expected_lines = [r"first.*Provision 2 entries",
                      r"second.*Provision 3 entries",
                      r"fourth.*Provision 1 entries",
                      r"Nothing"]
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    for line in expected_lines:
        print(line)
        assert re.search(line, stdout, re.MULTILINE)

    # destroy
    command = "mech destroy -f"
    expected = "Deleting"
    results = subprocess.run(command, cwd=test_dir, shell=True, capture_output=True)
    stdout = results.stdout.decode('utf-8')
    stderr = results.stderr.decode('utf-8')
    assert stderr == ''
    assert results.returncode == 0
    assert re.search(expected, stdout)

    # clean up at the end
    utils.cleanup_dir_and_vms_from_dir(test_dir)



# Copyright (c) 2020 Mike Kinney
"""Tests for VBoxManage class."""
from unittest.mock import patch, MagicMock


import mech.vbm


@patch('subprocess.Popen')
def test_vbm_run(mock_popen):
    """Test run method."""
    process_mock = MagicMock()
    attrs = {'communicate.return_value': ('output', 'error')}
    process_mock.configure_mock(**attrs)
    mock_popen.return_value = process_mock
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=False)
    got = vbm.run('list')
    assert got is None
    mock_popen.assert_called()


@patch('os.path.exists', return_value=True)
def test_vbm_installed(mock_path_exists):
    """Test installed."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage')
    assert vbm.installed()
    mock_path_exists.assert_called()
    assert vbm.get_executable() == '/bin/VBoxManage'


@patch('os.path.exists', return_value=False)
def test_vbm_executable_not_found(mock_path_exists):
    """Test installed."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage')
    assert not vbm.installed()
    mock_path_exists.assert_called()


def test_vbm_importvm():
    """Test importvm method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'import', '/tmp/first_tmp/some.ovf',
                '--vsys', '0', '--vmname', 'first', '--basefolder', '/tmp/first']
    got = vbm.importvm(path_to_ovf='/tmp/first_tmp/some.ovf',
                       name='first', base_folder='/tmp/first')
    assert got == expected


def test_vbm_start_headless():
    """Test start method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'startvm', 'first', '--type', 'headless']
    got = vbm.start(vmname='first')
    assert got == expected


def test_vbm__ip_no_ip():
    """Test _ip method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = None
    with patch.object(mech.vbm.VBoxManage, 'run', return_value='No value set!'):
        got = vbm._ip(vmname='first')
        assert got == expected


def test_vbm__ip():
    """Test _ip method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = '192.168.56.195'
    with patch.object(mech.vbm.VBoxManage, 'run', return_value='Value: 192.168.56.195'):
        got = vbm._ip(vmname='first')
        assert got == expected


def test_vbm_ip_with_wait():
    """Test ip method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = '192.168.56.195'
    with patch.object(mech.vbm.VBoxManage, '_ip', return_value='192.168.56.195'):
        got = vbm.ip(vmname='first', wait=True)
        assert got == expected


def test_vbm_ip_with_no_wait():
    """Test ip method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = '192.168.56.195'
    with patch.object(mech.vbm.VBoxManage, '_ip', return_value='192.168.56.195'):
        got = vbm.ip(vmname='first', wait=False)
        assert got == expected


def test_vbm_start_gui():
    """Test start method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'startvm', 'first', '--type', 'gui']
    got = vbm.start(vmname='first', gui=True)
    assert got == expected


def test_vbm_register():
    """Test register method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'registervm', '/tmp/some.ovf']
    got = vbm.register('/tmp/some.ovf')
    assert got == expected


def test_vbm_unregister():
    """Test unregister method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'unregistervm', 'first']
    got = vbm.unregister('first')
    assert got == expected


def test_vbm_stop():
    """Test stop method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'poweroff']
    got = vbm.stop('first')
    assert got == expected


def test_vbm_resume():
    """Test resume method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'resume']
    got = vbm.resume('first')
    assert got == expected


def test_vbm_reset():
    """Test reset method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'reset']
    got = vbm.reset('first')
    assert got == expected


def test_vbm_pause():
    """Test pause method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'controlvm', 'first', 'pause']
    got = vbm.pause('first')
    assert got == expected


def test_vbm_cpus_using_int():
    """Test cpus method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--cpus', '3']
    got = vbm.cpus('first', 3)
    assert got == expected


def test_vbm_cpus_using_string():
    """Test cpus method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--cpus', '3']
    got = vbm.cpus('first', "3")
    assert got == expected


def test_vbm_memory():
    """Test memory method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--memory', '2048']
    got = vbm.memory('first', 2048)
    assert got == expected


def test_vbm_sharedfolder_add():
    """Test sharedfolder_add method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'sharedfolder', 'add', 'first', '--name',
                'some_share', '--hostpath', '~/foo']
    got = vbm.sharedfolder_add('first', 'some_share', '~/foo')
    assert got == expected


def test_vbm_sharedfolder_remove():
    """Test sharedfolder_remove method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'sharedfolder', 'remove', 'first', '--name',
                'some_share']
    got = vbm.sharedfolder_remove('first', 'some_share')
    assert got == expected


def test_vbm_list_hostonly_ifs():
    """Test list_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'hostonlyifs']
    got = vbm.list_hostonly_ifs()
    assert got == expected


def test_vbm_create_hostonly():
    """Test create_hostonly method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = None
    with patch.object(mech.vbm.VBoxManage, 'list_hostonly_ifs', return_value='some output'):
        with patch.object(mech.vbm.VBoxManage, 'list_dhcpservers', return_value='some output'):
            with patch.object(mech.vbm.VBoxManage, 'create_hostonly_if', return_value=None):
                with patch.object(mech.vbm.VBoxManage, 'add_hostonly_dhcp', return_value=None):
                    got = vbm.create_hostonly()
                    assert got == expected


def test_vbm_create_hostonly_if():
    """Test create_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'hostonlyif', 'create']
    got = vbm.create_hostonly_if()
    assert got == expected


def test_vbm_remove_hostonly_if():
    """Test remove_hostonly_if method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'hostonlyif', 'remove', 'vboxnet0']
    got = vbm.remove_hostonly_if()
    assert got == expected


def test_vbm_add_hostonly_dhcp():
    """Test add_hostonly_dhcp method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'dhcpserver', 'add', '--ifname', 'vboxnet0',
                '--enable', '--ip', '192.168.56.1', '--netmask', '255.255.255.0',
                '--lower-ip', '192.168.56.100', '--upper-ip', '192.168.56.200']
    got = vbm.add_hostonly_dhcp()
    assert got == expected


def test_vbm_remove_hostonly_dhcp():
    """Test remove_hostonly_dhcp method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'dhcpserver', 'remove', '--network',
                'HostInterfaceNetworking-vboxnet0']
    got = vbm.remove_hostonly_dhcp()
    assert got == expected


def test_vbm_list_dhcpservers():
    """Test list_dhcpservers method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'dhcpservers']
    got = vbm.list_dhcpservers()
    assert got == expected


def test_vbm_hostonly():
    """Test hostonly method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--nic1', 'hostonly',
                '--hostonlyadapter1', 'vboxnet0']
    got = vbm.hostonly('first')
    assert got == expected


def test_vbm_bridged():
    """Test bridged method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'modifyvm', 'first', '--nic1', 'bridged',
                '--bridgeadapter1', 'en0']
    got = vbm.bridged('first')
    assert got == expected


def test_vbm_list():
    """Test list method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'list', 'vms']
    got = vbm.list()
    assert got == expected


def test_vbm_get_vm_info():
    """Test get_vm_info method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    expected = ['/bin/VBoxManage', 'showvminfo', 'first']
    got = vbm.get_vm_info('first')
    assert got == expected


def test_vbm_vm_state_running():
    """Test vm_state method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    output = """VT-x Unrestricted Exec.:     enabled
Paravirt. Provider:          Default
Effective Paravirt. Prov.:   KVM
State:                       running (since 2020-02-28T00:24:40.461000000)
Graphics Controller:         VBoxVGA

Monitor count:               1
"""
    expected = 'running'
    with patch.object(mech.vbm.VBoxManage, 'get_vm_info', return_value=output):
        got = vbm.vm_state('first')
        assert got == expected


def test_vbm_vm_state_unknown():
    """Test vm_state method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    output = None
    expected = None
    with patch.object(mech.vbm.VBoxManage, 'get_vm_info', return_value=output):
        got = vbm.vm_state('first')
        assert got == expected


def test_vbm_vm_state_powered_off():
    """Test vm_state method."""
    vbm = mech.vbm.VBoxManage(executable='/bin/VBoxManage', test_mode=True)
    output = """VT-x Unrestricted Exec.:     enabled
Paravirt. Provider:          Default
Effective Paravirt. Prov.:   KVM
State:                       powered off (since 2020-02-28T00:24:40.461000000)
Graphics Controller:         VBoxVGA

Monitor count:               1
"""
    expected = 'powered off'
    with patch.object(mech.vbm.VBoxManage, 'get_vm_info', return_value=output):
        got = vbm.vm_state('first')
        assert got == expected
